package model.dao;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;

import model.vo.Denominacio;
import model.vo.Vi;

public class ProducteHibernateDao implements ProducteDao{

	@SuppressWarnings("unchecked")
	public List<Denominacio> list() {
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		List<Denominacio> denominacions = null;
        try {
            denominacions = (List<Denominacio>)session
            	.createQuery("from Denominacio").list();
        } catch (HibernateException e) {
            e.printStackTrace();
            session.getTransaction().rollback();
        }
        session.getTransaction().commit();
        return denominacions;
	}
	
	public void addDenominacio(Denominacio denominacio) {
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx=null;
		try {
			tx = session.beginTransaction();
			session.save(denominacio);
			tx.commit();
		} catch (RuntimeException e) {
			if(tx != null) tx.rollback();
			throw e;
		} finally {
			session.close();
		}
	}
	
	public void updateDenominacio(Denominacio denominacio) {
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx=null;
		try {
			tx = session.beginTransaction();
			session.update(denominacio);
			tx.commit();
		} catch (RuntimeException e) {
			if(tx != null) tx.rollback();
			throw e;
		} finally {
			session.close();
		}
	}
	
	public Denominacio getDenominacio(int idDENOMINACIO) {
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		try {
			session.beginTransaction();
			Query q = session
					.createQuery("from Denominacio where ID_DENOMINACIO=:ID_DENOMINACIO");
			q.setInteger("ID_DENOMINACIO", idDENOMINACIO);
			return (Denominacio) q.uniqueResult();
		}finally {
			session.close();
		}
	}

	@SuppressWarnings("unchecked")
	public List<Vi> getProductsByDenominacio(int idDENOMINACIO) {
		Session session = HibernateUtil.getSessionFactory().openSession();
		session.beginTransaction();
		List<Vi> vins = null;
        try {
	 
            Query query = session.createQuery(
            		"from Vi where ID_DENOMINACIO =:idDENOMINACIO");
            query.setParameter("idDENOMINACIO", idDENOMINACIO);
            vins = query.list();
        } catch (HibernateException e) {
            e.printStackTrace();
            session.getTransaction().rollback();
        } 
        session.getTransaction().commit();
        return vins;
	}

	public void deleteDenominacio(int id_DENOMINACIO) {
	    Session session = HibernateUtil.getSessionFactory().getCurrentSession();
	    session.beginTransaction();
	    Denominacio denominacio = (Denominacio) session.load(Denominacio.class, id_DENOMINACIO);
	    if(null != denominacio) {
	    	session.delete(denominacio);
	    }
	    session.getTransaction().commit();
	}

	public void addVi(Vi vi) {
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx=null;
		try {
			tx = session.beginTransaction();
			session.save(vi);
			tx.commit();
		} catch (RuntimeException e) {
			if(tx != null) tx.rollback();
			throw e;
		} finally {
			session.close();
		}
	}

	public void updateVi(Vi vi) {
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx=null;
		try {
			tx = session.beginTransaction();
			session.update(vi);
			tx.commit();
		} catch (RuntimeException e) {
			if(tx != null) tx.rollback();
			throw e;
		} finally {
			session.close();
		}	
	}
	
	public void deleteVi(int idVI) {
	    Session session = HibernateUtil.getSessionFactory().getCurrentSession();
	    session.beginTransaction();
	    Vi vi = (Vi) session.load(Vi.class, idVI);
	    if(null != vi) {
	    	session.delete(vi);
	    }
	    session.getTransaction().commit();	
	}
	
	public Vi getVi(int idVI) {
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		try {
			session.beginTransaction();
			Query q = session
					.createQuery("from Vi where ID_VI=:ID_VI");
			q.setInteger("ID_VI", idVI);
			return (Vi) q.uniqueResult();
		}finally {
			session.close();
		}
	}

	@SuppressWarnings("unchecked")
	public List<Vi> getAllProducts() {
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		List<Vi> vins = null;
        try {
            vins = (List<Vi>)session
            	.createQuery("from Vi").list();
        } catch (HibernateException e) {
            e.printStackTrace();
            session.getTransaction().rollback();
        }
        session.getTransaction().commit();
        return vins;
	}

	@SuppressWarnings("unchecked")
	public List<Vi> getProductsByCategoria(String CATEGORIA) {
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		try {
			List<Vi> viList;
			session.beginTransaction();
			Query query = session.createQuery("from Vi where CATEGORIA =:CATEGORIA");
			query.setParameter("CATEGORIA", CATEGORIA);
			viList = query.list();
			return viList;
		} finally {
			session.close();
		}
	}

	@SuppressWarnings("unchecked")
	public List<Vi> getProductsByNom(String NOM) {
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		try {
			List<Vi> viList;
			session.beginTransaction();
			Query query = session.createQuery("from Vi where NOM  like :NOM");
			query.setParameter("NOM", "%" + NOM + "%");
			viList = query.list();
			return viList;
		} finally {
			session.close();
		}
	}

	@SuppressWarnings("unchecked")
	public List<Vi> getProductsByTipus(String TIPUS) {
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		try {
			List<Vi> viList;
			session.beginTransaction();
			Query query = session.createQuery("from Vi where TIPUS =:TIPUS");
			query.setParameter("TIPUS", TIPUS);
			viList = query.list();
			return viList;
		} finally {
			session.close();
		}
	}
	
	@SuppressWarnings("unchecked")
	public List<Vi> getProductsByCriteria (String find, String TIPUS, 
			String CATEGORIA, boolean description) {
		Session session = HibernateUtil.getSessionFactory().openSession();
		try {
			List<Vi> viList = null;
			Criteria criteria = session.createCriteria(Vi.class);
				if(description)
					criteria.add(Restrictions.like("DESCRIPCIO", "%" + find + "%"));
				else criteria.add(Restrictions.like("NOM", "%" + find + "%"));
				criteria.add(Restrictions.eq("TIPUS", TIPUS));
				criteria.add(Restrictions.eq("CATEGORIA", CATEGORIA));
			viList = criteria.list();
			return viList;
		} finally {
			session.close();
		}
	}
	
}
